@@ -23,18 +23,22 @@
 #include "gstspeexdec.h"
 #include "gstspeexenc.h"
 
+#include <gst/tag/tag.h>
+
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
 
-  if (!gst_element_register (plugin, "speexenc", GST_RANK_NONE,
-          GST_TYPE_SPEEXENC))
+  if (!gst_element_register (plugin, "speexenc", GST_RANK_PRIMARY,
+          GST_TYPE_SPEEX_ENC))
     return FALSE;
 
   if (!gst_element_register (plugin, "speexdec", GST_RANK_PRIMARY,
-          GST_TYPE_SPEEXDEC))
+          GST_TYPE_SPEEX_DEC))
     return FALSE;
 
+  gst_tag_register_musicbrainz_tags ();
+
   return TRUE;
 }
 
