@@ -21,6 +21,7 @@
 #  include "config.h"
 #endif
 
+#include <stdio.h>
 #include <string.h>
 
 #include <gst/rtp/gstrtpbuffer.h>
@@ -30,11 +31,13 @@ GST_DEBUG_CATEGORY_STATIC (rtph264depay_debug);
 #define GST_CAT_DEFAULT (rtph264depay_debug)
 
 #define DEFAULT_BYTE_STREAM	TRUE
+#define DEFAULT_ACCESS_UNIT	FALSE
 
 enum
 {
   PROP_0,
   PROP_BYTE_STREAM,
+  PROP_ACCESS_UNIT,
   PROP_LAST
 };
 
@@ -42,13 +45,6 @@ enum
 /* 3 zero bytes syncword */
 static const guint8 sync_bytes[] = { 0, 0, 0, 1 };
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_h264depay_details =
-GST_ELEMENT_DETAILS ("RTP H264 depayloader",
-    "Codec/Depayloader/Network",
-    "Extracts H264 video from RTP packets (RFC 3984)",
-    "Wim Taymans <wim.taymans@gmail.com>");
-
 static GstStaticPadTemplate gst_rtp_h264_depay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -110,7 +106,10 @@ gst_rtp_h264_depay_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_h264_depay_sink_template));
 
-  gst_element_class_set_details (element_class, &gst_rtp_h264depay_details);
+  gst_element_class_set_details_simple (element_class, "RTP H264 depayloader",
+      "Codec/Depayloader/Network",
+      "Extracts H264 video from RTP packets (RFC 3984)",
+      "Wim Taymans <wim.taymans@gmail.com>");
 }
 
 static void
@@ -133,6 +132,10 @@ gst_rtp_h264_depay_class_init (GstRtpH264DepayClass * klass)
       g_param_spec_boolean ("byte-stream", "Byte Stream",
           "Generate byte stream format of NALU", DEFAULT_BYTE_STREAM,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_ACCESS_UNIT,
+      g_param_spec_boolean ("access-unit", "Access Unit",
+          "Merge NALU into AU (picture)", DEFAULT_ACCESS_UNIT,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state = gst_rtp_h264_depay_change_state;
 
@@ -148,7 +151,9 @@ gst_rtp_h264_depay_init (GstRtpH264Depay * rtph264depay,
     GstRtpH264DepayClass * klass)
 {
   rtph264depay->adapter = gst_adapter_new ();
+  rtph264depay->picture_adapter = gst_adapter_new ();
   rtph264depay->byte_stream = DEFAULT_BYTE_STREAM;
+  rtph264depay->merge = DEFAULT_ACCESS_UNIT;
 }
 
 static void
@@ -162,6 +167,7 @@ gst_rtp_h264_depay_finalize (GObject * object)
     gst_buffer_unref (rtph264depay->codec_data);
 
   g_object_unref (rtph264depay->adapter);
+  g_object_unref (rtph264depay->picture_adapter);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -178,6 +184,9 @@ gst_rtp_h264_depay_set_property (GObject * object, guint prop_id,
     case PROP_BYTE_STREAM:
       rtph264depay->byte_stream = g_value_get_boolean (value);
       break;
+    case PROP_ACCESS_UNIT:
+      rtph264depay->merge = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -196,60 +205,15 @@ gst_rtp_h264_depay_get_property (GObject * object, guint prop_id,
     case PROP_BYTE_STREAM:
       g_value_set_boolean (value, rtph264depay->byte_stream);
       break;
+    case PROP_ACCESS_UNIT:
+      g_value_set_boolean (value, rtph264depay->merge);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
-static const guint8 a2bin[256] = {
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 62, 64, 64, 64, 63,
-  52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 64, 64, 64, 64, 64, 64,
-  64, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
-  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 64, 64, 64, 64, 64,
-  64, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
-  41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,
-  64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64
-};
-
-static guint
-decode_base64 (gchar * in, guint8 * out)
-{
-  guint8 v1, v2;
-  guint len = 0;
-
-  v1 = a2bin[(gint) * in];
-  while (v1 <= 63) {
-    /* read 4 bytes, write 3 bytes, invalid base64 are zeroes */
-    v2 = a2bin[(gint) * ++in];
-    *out++ = (v1 << 2) | ((v2 & 0x3f) >> 4);
-    v1 = (v2 > 63 ? 64 : a2bin[(gint) * ++in]);
-    *out++ = (v2 << 4) | ((v1 & 0x3f) >> 2);
-    v2 = (v1 > 63 ? 64 : a2bin[(gint) * ++in]);
-    *out++ = (v1 << 6) | (v2 & 0x3f);
-    v1 = (v2 > 63 ? 64 : a2bin[(gint) * ++in]);
-    len += 3;
-  }
-  /* move to '\0' */
-  while (*in != '\0')
-    in++;
-
-  /* subtract padding */
-  while (len > 0 && *--in == '=')
-    len--;
-
-  return len;
-}
-
 static gboolean
 gst_rtp_h264_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 {
@@ -296,10 +260,16 @@ gst_rtp_h264_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
     b64 = GST_BUFFER_DATA (codec_data);
     total = 0;
     for (i = 0; params[i]; i++) {
-      GST_DEBUG_OBJECT (depayload, "decoding param %d", i);
+      guint save = 0;
+      gint state = 0;
+
+      GST_DEBUG_OBJECT (depayload, "decoding param %d (%s)", i, params[i]);
       memcpy (b64, sync_bytes, sizeof (sync_bytes));
       b64 += sizeof (sync_bytes);
-      len = decode_base64 (params[i], b64);
+      len =
+          g_base64_decode_step (params[i], strlen (params[i]), b64, &state,
+          &save);
+      GST_DEBUG_OBJECT (depayload, "decoded %d bytes", len);
       total += len + sizeof (sync_bytes);
       b64 += len;
     }
@@ -318,9 +288,8 @@ gst_rtp_h264_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
     guint len, num_sps, num_pps;
     gint i;
     guint8 *data;
-    guint32 profile_id;
 
-    if (ps == NULL || profile == NULL)
+    if (ps == NULL)
       goto incomplete_caps;
 
     params = g_strsplit (ps, ",", 0);
@@ -335,12 +304,16 @@ gst_rtp_h264_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
     /* start with 7 bytes header */
     len = 7;
     for (i = 0; params[i]; i++) {
-      gint nal_len;
+      gsize nal_len;
       guint8 *nalp;
+      guint save = 0;
+      gint state = 0;
 
       nal_len = strlen (params[i]);
       nalp = g_malloc (nal_len + 2);
-      nal_len = decode_base64 (params[i], nalp + 2);
+
+      nal_len =
+          g_base64_decode_step (params[i], nal_len, nalp + 2, &state, &save);
       nalp[0] = (nal_len >> 8) & 0xff;
       nalp[1] = nal_len & 0xff;
       len += nal_len + 2;
@@ -356,16 +329,31 @@ gst_rtp_h264_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
     }
     g_strfreev (params);
 
+    if (num_sps == 0 || (GST_READ_UINT16_BE (sps[0]) < 3) || num_pps == 0) {
+      g_strfreev ((gchar **) pps);
+      g_strfreev ((gchar **) sps);
+      goto incomplete_caps;
+    }
+
     codec_data = gst_buffer_new_and_alloc (len);
     data = GST_BUFFER_DATA (codec_data);
 
     /* 8 bits version == 1 */
     *data++ = 1;
-    /* hex: AVCProfileIndication:8 | profile_compat:8 | AVCLevelIndication:8 */
-    sscanf (profile, "%6x", &profile_id);
-    *data++ = (profile_id >> 16) & 0xff;
-    *data++ = (profile_id >> 8) & 0xff;
-    *data++ = profile_id & 0xff;
+    if (profile) {
+      guint32 profile_id;
+
+      /* hex: AVCProfileIndication:8 | profile_compat:8 | AVCLevelIndication:8 */
+      sscanf (profile, "%6x", &profile_id);
+      *data++ = (profile_id >> 16) & 0xff;
+      *data++ = (profile_id >> 8) & 0xff;
+      *data++ = profile_id & 0xff;
+    } else {
+      /* extract from SPS */
+      *data++ = sps[0][3];
+      *data++ = sps[0][4];
+      *data++ = sps[0][5];
+    }
     /* 6 bits reserved | 2 bits lengthSizeMinusOn */
     *data++ = 0xff;
     /* 3 bits reserved | 5 bits numOfSequenceParameterSets */
@@ -411,8 +399,68 @@ incomplete_caps:
   }
 }
 
-/* FIXME, non-bytestream handling is freaking out ffmpeg. Apparently we need to
- * group all NAL units belonging to one frame together */
+static GstBuffer *
+gst_rtp_h264_depay_push_nal (GstRtpH264Depay * rtph264depay, GstBuffer * nal,
+    GstClockTime timestamp)
+{
+  gint nal_type;
+  guint8 *data;
+  GstBuffer *outbuf = NULL;
+  guint outsize;
+  gboolean start = FALSE;
+
+  data = GST_BUFFER_DATA (nal);
+
+  /* caller ensures number of bytes available */
+  nal_type = data[4] & 0x1f;
+  GST_DEBUG_OBJECT (rtph264depay, "collecting NALU type %d", nal_type);
+  /* consider a coded slices (IDR or not) to start a picture,
+   * (so ending the previous one) if first_mb_in_slice == 0
+   * (non-0 is part of previous one) */
+  /* NOTE this is not entirely according to Access Unit specs in 7.4.1.2.4,
+   * but in practice it works in sane cases, needs not much parsing,
+   * and also works with broken frame_num in NAL (where spec-wise would fail) */
+  if (nal_type == 1 || nal_type == 2 || nal_type == 5) {
+    if (rtph264depay->picture_start) {
+      if (data[5] & 0x80) {
+        /* first_mb_in_slice == 0 */
+        rtph264depay->picture_complete = TRUE;
+        start = TRUE;
+      }
+      /* otherwise simply collect */
+    } else {
+      rtph264depay->picture_start = TRUE;
+    }
+  } else if (nal_type >= 6 && nal_type <= 9) {
+    /* SEI, SPS, PPS, AU terminate picture */
+    if (rtph264depay->picture_start)
+      rtph264depay->picture_complete = TRUE;
+    /* otherwise simply collect */
+  }
+
+  if (rtph264depay->picture_complete) {
+    outsize = gst_adapter_available (rtph264depay->picture_adapter);
+    outbuf = gst_adapter_take_buffer (rtph264depay->picture_adapter, outsize);
+    rtph264depay->picture_complete = FALSE;
+    rtph264depay->picture_start = start;
+
+    if (G_UNLIKELY (rtph264depay->codec_data)) {
+      outbuf = gst_buffer_join (rtph264depay->codec_data, outbuf);
+      rtph264depay->codec_data = NULL;
+    }
+
+    GST_BUFFER_TIMESTAMP (outbuf) = rtph264depay->last_ts;
+    if (GST_CLOCK_TIME_IS_VALID (timestamp) &&
+        GST_CLOCK_TIME_IS_VALID (rtph264depay->last_ts))
+      GST_BUFFER_DURATION (outbuf) = timestamp - rtph264depay->last_ts;
+  }
+
+  rtph264depay->last_ts = timestamp;
+  gst_adapter_push (rtph264depay->picture_adapter, nal);
+
+  return outbuf;
+}
+
 static GstBuffer *
 gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 {
@@ -435,6 +483,11 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     guint8 nal_ref_idc;
     guint8 *outdata;
     guint outsize, nalu_size;
+    GstClockTime ts;
+
+    /* baseclass already did this, would be nice not to have to redo ... */
+    ts = GST_BUFFER_TIMESTAMP (buf);
+    ts = gst_segment_to_running_time (&depayload->segment, GST_FORMAT_TIME, ts);
 
     payload_len = gst_rtp_buffer_get_payload_len (buf);
     payload = gst_rtp_buffer_get_payload (buf);
@@ -478,7 +531,7 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
         rtph264depay->wait_start = FALSE;
 
         /* prepend codec_data */
-        if (rtph264depay->codec_data) {
+        if (!rtph264depay->merge && rtph264depay->codec_data) {
           gst_adapter_push (rtph264depay->adapter, rtph264depay->codec_data);
           rtph264depay->codec_data = NULL;
         }
@@ -514,18 +567,28 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
           outdata += sizeof (sync_bytes);
           memcpy (outdata, payload, nalu_size);
 
-          gst_adapter_push (rtph264depay->adapter, outbuf);
+          if (!rtph264depay->merge)
+            gst_adapter_push (rtph264depay->adapter, outbuf);
+          else {
+            outbuf = gst_rtp_h264_depay_push_nal (rtph264depay, outbuf, ts);
+            if (outbuf) {
+              gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
+              /* already timestamped this buffer, baseclass need not bother */
+              gst_base_rtp_depayload_push (depayload, outbuf);
+            }
+          }
 
           payload += nalu_size;
           payload_len -= nalu_size;
         }
 
-        outsize = gst_adapter_available (rtph264depay->adapter);
-        outbuf = gst_adapter_take_buffer (rtph264depay->adapter, outsize);
-
-        gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
-
-        return outbuf;
+        if (!rtph264depay->merge) {
+          outsize = gst_adapter_available (rtph264depay->adapter);
+          outbuf = gst_adapter_take_buffer (rtph264depay->adapter, outsize);
+          gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
+          return outbuf;
+        }
+        break;
       }
       case 26:
         /* MTAP16    Multi-time aggregation packet      5.7.2 */
@@ -618,15 +681,22 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
             outdata[2] = (outsize >> 8);
             outdata[3] = (outsize);
           }
-          gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
 
           /* push codec_data first */
-          if (rtph264depay->codec_data) {
+          if (!rtph264depay->merge && rtph264depay->codec_data) {
             gst_buffer_set_caps (rtph264depay->codec_data,
                 GST_PAD_CAPS (depayload->srcpad));
             gst_base_rtp_depayload_push (depayload, rtph264depay->codec_data);
             rtph264depay->codec_data = NULL;
           }
+
+          if (rtph264depay->merge) {
+            outbuf = gst_rtp_h264_depay_push_nal (rtph264depay, outbuf, ts);
+            if (!outbuf)
+              break;
+          }
+
+          gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
           return outbuf;
         }
         break;
@@ -651,15 +721,21 @@ gst_rtp_h264_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
         outdata += sizeof (sync_bytes);
         memcpy (outdata, payload, nalu_size);
 
-        gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
-
         /* push codec_data first */
-        if (rtph264depay->codec_data) {
+        if (!rtph264depay->merge && rtph264depay->codec_data) {
           gst_buffer_set_caps (rtph264depay->codec_data,
               GST_PAD_CAPS (depayload->srcpad));
           gst_base_rtp_depayload_push (depayload, rtph264depay->codec_data);
           rtph264depay->codec_data = NULL;
         }
+
+        if (rtph264depay->merge) {
+          outbuf = gst_rtp_h264_depay_push_nal (rtph264depay, outbuf, ts);
+          if (!outbuf)
+            break;
+        }
+
+        gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
         return outbuf;
       }
     }
@@ -702,6 +778,9 @@ gst_rtp_h264_depay_change_state (GstElement * element,
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       gst_adapter_clear (rtph264depay->adapter);
       rtph264depay->wait_start = TRUE;
+      gst_adapter_clear (rtph264depay->picture_adapter);
+      rtph264depay->picture_start = FALSE;
+      rtph264depay->picture_complete = FALSE;
       break;
     default:
       break;
