@@ -26,13 +26,6 @@
 #include <string.h>
 #include "gstrtpmp1sdepay.h"
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_mp1sdepay_details =
-GST_ELEMENT_DETAILS ("RTP packet depayloader",
-    "Codec/Depayloader/Network",
-    "Extracts MPEG1 System Streams from RTP packets (RFC 3555)",
-    "Wim Taymans <wim.taymans@gmail.com>");
-
 /* RtpMP1SDepay signals and args */
 enum
 {
@@ -77,15 +70,6 @@ static gboolean gst_rtp_mp1s_depay_setcaps (GstBaseRTPDepayload * depayload,
 static GstBuffer *gst_rtp_mp1s_depay_process (GstBaseRTPDepayload * depayload,
     GstBuffer * buf);
 
-static void gst_rtp_mp1s_depay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_rtp_mp1s_depay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
-
-static GstStateChangeReturn gst_rtp_mp1s_depay_change_state (GstElement *
-    element, GstStateChange transition);
-
-
 static void
 gst_rtp_mp1s_depay_base_init (gpointer klass)
 {
@@ -96,30 +80,21 @@ gst_rtp_mp1s_depay_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_mp1s_depay_sink_template));
 
-  gst_element_class_set_details (element_class, &gst_rtp_mp1sdepay_details);
+  gst_element_class_set_details_simple (element_class,
+      "RTP MPEG1 System Stream depayloader", "Codec/Depayloader/Network",
+      "Extracts MPEG1 System Streams from RTP packets (RFC 3555)",
+      "Wim Taymans <wim.taymans@gmail.com>");
 }
 
 static void
 gst_rtp_mp1s_depay_class_init (GstRtpMP1SDepayClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseRTPDepayloadClass *gstbasertpdepayload_class;
 
-  gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
-
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gstbasertpdepayload_class->process = gst_rtp_mp1s_depay_process;
   gstbasertpdepayload_class->set_caps = gst_rtp_mp1s_depay_setcaps;
-
-  gobject_class->set_property = gst_rtp_mp1s_depay_set_property;
-  gobject_class->get_property = gst_rtp_mp1s_depay_get_property;
-
-  gstelement_class->change_state = gst_rtp_mp1s_depay_change_state;
 }
 
 static void
@@ -133,111 +108,33 @@ gst_rtp_mp1s_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 {
   GstCaps *srccaps;
   GstStructure *structure;
-  GstRtpMP1SDepay *rtpmp1sdepay;
-  gint clock_rate = 90000;      /* default */
-
-  rtpmp1sdepay = GST_RTP_MP1S_DEPAY (depayload);
+  gint clock_rate;
+  gboolean res;
 
   structure = gst_caps_get_structure (caps, 0);
-  gst_structure_get_int (structure, "clock-rate", &clock_rate);
+  if (!gst_structure_get_int (structure, "clock-rate", &clock_rate))
+    clock_rate = 90000;         /* default */
   depayload->clock_rate = clock_rate;
 
   srccaps = gst_caps_new_simple ("video/mpeg",
       "systemstream", G_TYPE_BOOLEAN, TRUE, NULL);
-  gst_pad_set_caps (GST_BASE_RTP_DEPAYLOAD_SRCPAD (depayload), srccaps);
+  res = gst_pad_set_caps (GST_BASE_RTP_DEPAYLOAD_SRCPAD (depayload), srccaps);
   gst_caps_unref (srccaps);
 
-  return TRUE;
+  return res;
 }
 
 static GstBuffer *
 gst_rtp_mp1s_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 {
-  GstRtpMP1SDepay *rtpmp1sdepay;
   GstBuffer *outbuf;
-  gint payload_len;
 
-  rtpmp1sdepay = GST_RTP_MP1S_DEPAY (depayload);
-
-  if (G_UNLIKELY (!gst_rtp_buffer_validate (buf)))
-    goto bad_packet;
-
-  payload_len = gst_rtp_buffer_get_payload_len (buf);
-
-  outbuf = gst_rtp_buffer_get_payload_subbuffer (buf, 0, -1);
-  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
+  outbuf = gst_rtp_buffer_get_payload_buffer (buf);
 
   GST_DEBUG ("gst_rtp_mp1s_depay_chain: pushing buffer of size %d",
       GST_BUFFER_SIZE (outbuf));
 
   return outbuf;
-
-  /* ERRORS */
-bad_packet:
-  {
-    GST_ELEMENT_WARNING (rtpmp1sdepay, STREAM, DECODE,
-        (NULL), ("Packet did not validate"));
-    return NULL;
-  }
-}
-
-static void
-gst_rtp_mp1s_depay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstRtpMP1SDepay *rtpmp1sdepay;
-
-  rtpmp1sdepay = GST_RTP_MP1S_DEPAY (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_rtp_mp1s_depay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstRtpMP1SDepay *rtpmp1sdepay;
-
-  rtpmp1sdepay = GST_RTP_MP1S_DEPAY (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static GstStateChangeReturn
-gst_rtp_mp1s_depay_change_state (GstElement * element,
-    GstStateChange transition)
-{
-  GstRtpMP1SDepay *rtpmp1sdepay;
-  GstStateChangeReturn ret;
-
-  rtpmp1sdepay = GST_RTP_MP1S_DEPAY (element);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_NULL_TO_READY:
-      break;
-    case GST_STATE_CHANGE_READY_TO_PAUSED:
-      break;
-    default:
-      break;
-  }
-
-  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_READY_TO_NULL:
-      break;
-    default:
-      break;
-  }
-  return ret;
 }
 
 gboolean
