@@ -19,31 +19,23 @@
 
 /**
  * SECTION:element-smpte
- * @short_description: Takes to video frames and applies an SMPTE transition 
- * effect on them.
  *
- * <refsect2>
- * <para>
  * smpte can accept I420 video streams with the same width, height and
  * framerate. The two incomming buffers are blended together using an effect
  * specific alpha mask. 
- * </para>
- * <para>
- * The depth property defines the presision in bits of the mask. A higher
- * presision will create a mask with smoother gradients in order to avoid
+ *
+ * The #GstSmpte:depth property defines the presision in bits of the mask. A
+ * higher presision will create a mask with smoother gradients in order to avoid
  * banding.
- * </para>
+ *
+ * <refsect2>
  * <title>Sample pipelines</title>
- * <para>
- * Here is a pipeline to demonstrate the smpte transition :
- * <programlisting>
- * gst-launch -v videotestsrc pattern=1 ! smpte name=s border=20000 type=234
- * duration=2000000000 ! ffmpegcolorspace ! ximagesink videotestsrc ! s.
- * </programlisting>
- * This shows a pinwheel transition a from a snow videotestsrc to an smpte
+ * |[
+ * gst-launch -v videotestsrc pattern=1 ! smpte name=s border=20000 type=234 duration=2000000000 ! ffmpegcolorspace ! ximagesink videotestsrc ! s.
+ * ]| A pipeline to demonstrate the smpte transition.
+ * It shows a pinwheel transition a from a snow videotestsrc to an smpte
  * pattern videotestsrc. The transition will take 2 seconds to complete. The
  * edges of the transition are smoothed with a 20000 big border.
- * </para>
  * </refsect2>
  */
 
@@ -58,13 +50,6 @@
 GST_DEBUG_CATEGORY_STATIC (gst_smpte_debug);
 #define GST_CAT_DEFAULT gst_smpte_debug
 
-/* elementfactory information */
-static const GstElementDetails smpte_details =
-GST_ELEMENT_DETAILS ("SMPTE transitions",
-    "Filter/Editor/Video",
-    "Apply the standard SMPTE transitions on video images",
-    "Wim Taymans <wim.taymans@chello.be>");
-
 static GstStaticPadTemplate gst_smpte_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -114,6 +99,17 @@ enum
   PROP_LAST,
 };
 
+#define I420_Y_ROWSTRIDE(width) (GST_ROUND_UP_4(width))
+#define I420_U_ROWSTRIDE(width) (GST_ROUND_UP_8(width)/2)
+#define I420_V_ROWSTRIDE(width) ((GST_ROUND_UP_8(I420_Y_ROWSTRIDE(width)))/2)
+
+#define I420_Y_OFFSET(w,h) (0)
+#define I420_U_OFFSET(w,h) (I420_Y_OFFSET(w,h)+(I420_Y_ROWSTRIDE(w)*GST_ROUND_UP_2(h)))
+#define I420_V_OFFSET(w,h) (I420_U_OFFSET(w,h)+(I420_U_ROWSTRIDE(w)*GST_ROUND_UP_2(h)/2))
+
+#define I420_SIZE(w,h)     (I420_V_OFFSET(w,h)+(I420_V_ROWSTRIDE(w)*GST_ROUND_UP_2(h)/2))
+
+
 #define GST_TYPE_SMPTE_TRANSITION_TYPE (gst_smpte_transition_type_get_type())
 static GType
 gst_smpte_transition_type_get_type (void)
@@ -135,8 +131,9 @@ gst_smpte_transition_type_get_type (void)
       definitions = g_list_next (definitions);
 
       smpte_transitions[i].value = definition->type;
-      smpte_transitions[i].value_nick = definition->short_name;
-      smpte_transitions[i].value_name = definition->long_name;
+      /* older GLib versions have the two fields as non-const, hence the cast */
+      smpte_transitions[i].value_nick = (gchar *) definition->short_name;
+      smpte_transitions[i].value_name = (gchar *) definition->long_name;
 
       i++;
     }
@@ -151,6 +148,7 @@ gst_smpte_transition_type_get_type (void)
 static void gst_smpte_class_init (GstSMPTEClass * klass);
 static void gst_smpte_base_init (GstSMPTEClass * klass);
 static void gst_smpte_init (GstSMPTE * smpte);
+static void gst_smpte_finalize (GstSMPTE * smpte);
 
 static GstFlowReturn gst_smpte_collected (GstCollectPads * pads,
     GstSMPTE * smpte);
@@ -202,7 +200,10 @@ gst_smpte_base_init (GstSMPTEClass * klass)
       gst_static_pad_template_get (&gst_smpte_sink2_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_smpte_src_template));
-  gst_element_class_set_details (element_class, &smpte_details);
+  gst_element_class_set_details_simple (element_class, "SMPTE transitions",
+      "Filter/Editor/Video",
+      "Apply the standard SMPTE transitions on video images",
+      "Wim Taymans <wim.taymans@chello.be>");
 }
 
 static void
@@ -218,6 +219,7 @@ gst_smpte_class_init (GstSMPTEClass * klass)
 
   gobject_class->set_property = gst_smpte_set_property;
   gobject_class->get_property = gst_smpte_get_property;
+  gobject_class->finalize = (GObjectFinalizeFunc) gst_smpte_finalize;
 
   _gst_mask_init ();
 
@@ -244,18 +246,19 @@ gst_smpte_class_init (GstSMPTEClass * klass)
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_smpte_change_state);
 }
 
-/*                        wht  yel  cya  grn  mag  red  blu  blk   -I    Q */
-static int y_colors[] = { 255, 226, 179, 150, 105, 76, 29, 16, 16, 0 };
-static int u_colors[] = { 128, 0, 170, 46, 212, 85, 255, 128, 0, 128 };
-static int v_colors[] = { 128, 155, 0, 21, 235, 255, 107, 128, 128, 255 };
+/*                              wht  yel  cya  grn  mag  red  blu  blk   -I    Q */
+static const int y_colors[] = { 255, 226, 179, 150, 105, 76, 29, 16, 16, 0 };
+static const int u_colors[] = { 128, 0, 170, 46, 212, 85, 255, 128, 0, 128 };
+static const int v_colors[] = { 128, 155, 0, 21, 235, 255, 107, 128, 128, 255 };
 
 static void
 fill_i420 (guint8 * data, gint width, gint height, gint color)
 {
-  gint size = width * height, size4 = size >> 2;
+  gint size = I420_Y_ROWSTRIDE (width) * GST_ROUND_UP_2 (height);
+  gint size4 = size >> 2;
   guint8 *yp = data;
-  guint8 *up = data + size;
-  guint8 *vp = data + size + size4;
+  guint8 *up = data + I420_U_OFFSET (width, height);
+  guint8 *vp = data + I420_V_OFFSET (width, height);
 
   memset (yp, y_colors[color], size);
   memset (up, u_colors[color], size4);
@@ -268,6 +271,13 @@ gst_smpte_update_mask (GstSMPTE * smpte, gint type, gint depth, gint width,
 {
   GstMask *newmask;
 
+  if (smpte->mask) {
+    if (smpte->type == type &&
+        smpte->depth == depth &&
+        smpte->width == width && smpte->height == height)
+      return TRUE;
+  }
+
   newmask = gst_mask_factory_new (type, depth, width, height);
   if (newmask) {
     if (smpte->mask) {
@@ -322,12 +332,18 @@ gst_smpte_init (GstSMPTE * smpte)
 {
   smpte->sinkpad1 =
       gst_pad_new_from_static_template (&gst_smpte_sink1_template, "sink1");
-  gst_pad_set_setcaps_function (smpte->sinkpad1, gst_smpte_setcaps);
+  gst_pad_set_setcaps_function (smpte->sinkpad1,
+      GST_DEBUG_FUNCPTR (gst_smpte_setcaps));
+  gst_pad_set_getcaps_function (smpte->sinkpad1,
+      GST_DEBUG_FUNCPTR (gst_pad_proxy_getcaps));
   gst_element_add_pad (GST_ELEMENT (smpte), smpte->sinkpad1);
 
   smpte->sinkpad2 =
       gst_pad_new_from_static_template (&gst_smpte_sink2_template, "sink2");
-  gst_pad_set_setcaps_function (smpte->sinkpad2, gst_smpte_setcaps);
+  gst_pad_set_setcaps_function (smpte->sinkpad2,
+      GST_DEBUG_FUNCPTR (gst_smpte_setcaps));
+  gst_pad_set_getcaps_function (smpte->sinkpad2,
+      GST_DEBUG_FUNCPTR (gst_pad_proxy_getcaps));
   gst_element_add_pad (GST_ELEMENT (smpte), smpte->sinkpad2);
 
   smpte->srcpad =
@@ -353,6 +369,16 @@ gst_smpte_init (GstSMPTE * smpte)
   smpte->fps_denom = 1;
 }
 
+static void
+gst_smpte_finalize (GstSMPTE * smpte)
+{
+  if (smpte->collect) {
+    gst_object_unref (smpte->collect);
+  }
+
+  G_OBJECT_CLASS (parent_class)->finalize ((GObject *) smpte);
+}
+
 static void
 gst_smpte_reset (GstSMPTE * smpte)
 {
@@ -411,7 +437,10 @@ gst_smpte_collected (GstCollectPads * pads, GstSMPTE * smpte)
   GstBuffer *in1 = NULL, *in2 = NULL;
   GSList *collected;
 
-  if (smpte->fps_num == 0)
+  if (G_UNLIKELY (smpte->fps_num == 0))
+    goto not_negotiated;
+
+  if (!GST_PAD_CAPS (smpte->sinkpad1) || !GST_PAD_CAPS (smpte->sinkpad2))
     goto not_negotiated;
 
   ts = gst_util_uint64_scale_int (smpte->position * GST_SECOND,
@@ -430,25 +459,28 @@ gst_smpte_collected (GstCollectPads * pads, GstSMPTE * smpte)
 
   if (in1 == NULL) {
     /* if no input, make picture black */
-    in1 = gst_buffer_new_and_alloc (smpte->width * smpte->height * 3);
+    in1 = gst_buffer_new_and_alloc (I420_SIZE (smpte->width, smpte->height));
     fill_i420 (GST_BUFFER_DATA (in1), smpte->width, smpte->height, 7);
   }
   if (in2 == NULL) {
     /* if no input, make picture white */
-    in2 = gst_buffer_new_and_alloc (smpte->width * smpte->height * 3);
+    in2 = gst_buffer_new_and_alloc (I420_SIZE (smpte->width, smpte->height));
     fill_i420 (GST_BUFFER_DATA (in2), smpte->width, smpte->height, 0);
   }
 
+  if (GST_BUFFER_SIZE (in1) != GST_BUFFER_SIZE (in2))
+    goto input_formats_do_not_match;
+
   if (smpte->position < smpte->end_position) {
-    outbuf = gst_buffer_new_and_alloc (smpte->width * smpte->height * 3);
+    outbuf = gst_buffer_new_and_alloc (I420_SIZE (smpte->width, smpte->height));
 
     /* set caps if not done yet */
     if (!GST_PAD_CAPS (smpte->srcpad)) {
       GstCaps *caps;
 
       caps =
-          gst_caps_copy (gst_static_caps_get (&gst_smpte_src_template.
-              static_caps));
+          gst_caps_copy (gst_static_caps_get
+          (&gst_smpte_src_template.static_caps));
       gst_caps_set_simple (caps, "width", G_TYPE_INT, smpte->width, "height",
           G_TYPE_INT, smpte->height, "framerate", GST_TYPE_FRACTION,
           smpte->fps_num, smpte->fps_denom, NULL);
@@ -493,6 +525,13 @@ not_negotiated:
         ("No input format negotiated"));
     return GST_FLOW_NOT_NEGOTIATED;
   }
+input_formats_do_not_match:
+  {
+    GST_ELEMENT_ERROR (smpte, CORE, NEGOTIATION, (NULL),
+        ("input formats don't match: %" GST_PTR_FORMAT " vs. %" GST_PTR_FORMAT,
+            GST_PAD_CAPS (smpte->sinkpad1), GST_PAD_CAPS (smpte->sinkpad2)));
+    return GST_FLOW_ERROR;
+  }
 }
 
 static void
@@ -589,17 +628,11 @@ gst_smpte_change_state (GstElement * element, GstStateChange transition)
   return ret;
 }
 
-static gboolean
-plugin_init (GstPlugin * plugin)
+gboolean
+gst_smpte_plugin_init (GstPlugin * plugin)
 {
   GST_DEBUG_CATEGORY_INIT (gst_smpte_debug, "smpte", 0,
       "SMPTE transition effect");
 
   return gst_element_register (plugin, "smpte", GST_RANK_NONE, GST_TYPE_SMPTE);
 }
-
-GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
-    GST_VERSION_MINOR,
-    "smpte",
-    "Apply the standard SMPTE transitions on video images",
-    plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
